import re

from shell import shell
import click

from .fs_abc import FilesystemAbstract
from .fs_config import FilesystemConfig
from .sshfs import SSHFS
from .smbfs import SMBFS

mount_line_pattern = re.compile(r"(.*) on (.*) \(.*\)$")


def create_fs(config: FilesystemConfig) -> FilesystemAbstract:
    if config.filesystem == "sshfs":
        return SSHFS(config)
    if config.filesystem == "smbfs":
        return SMBFS(config)
    else:
        raise ValueError(f"{config.filesystem} is not supported")


def ls_mounts(config_dir: str):
    config_files = FilesystemConfig.ls_files(config_dir)
    remotes: dict[str, str] = {}

    for file in config_files:
        filesystem = create_fs(FilesystemConfig.from_file(file))
        remotes[filesystem.format_remote()] = filesystem.mount_point
        # remotes.append(filesystem.format_remote(), filesystem.mount_point)

    mount_lines = shell("mount").output()
    for line in mount_lines:
        matches = re.findall(mount_line_pattern, line)[0]
        if not matches:
            continue
        remote, mount_point = matches
        if remote in remotes:
            click.echo(f"{remote} on {mount_point}")



import errno
import json
import os

from pathvalidate import sanitize_filename


class FilesystemConfig:
    def __init__(
        self,
        filesystem="",
        mount_point="",
        remote="",
        hostname="",
        user="",
        dir="",
        option=[],
    ):
        self.filesystem = filesystem
        self.mount_point = mount_point
        self.remote = remote
        self.hostname = hostname
        self.user = user
        self.dir = dir
        # NOTE: missing s on purpose
        self.options = option

    def args(self):
        return (
            self.hostname,
            self.mount_point,
            {"user": self.user, "dir": self.dir, "options": self.options},
        )

    def save(self, name, config_dir):
        self.name = name
        basename = sanitize_filename(f"{name}.json")
        filename = os.path.join(config_dir, basename)
        os.makedirs(config_dir, exist_ok=True)
        json.dump(self.__dict__, open(filename, "w"))

    def load(self, name, config_dir):
        basename = f"{name}.json"
        filename = os.path.join(config_dir, basename)
        self.__dict__.update(json.load(open(filename, "r")))

    @classmethod
    def from_file(cls, filename):
        fs_config = cls()
        fs_config.__dict__.update(json.load(open(filename, "r")))
        return fs_config

    @staticmethod
    def ls_configs(config_dir) -> list[tuple[str, str]]:
        names = []
        for file in os.listdir(config_dir):
            if file.endswith(".json"):
                config_filename = os.path.join(config_dir, file)
                config_json = json.load(open(config_filename, "r"))
                names.append((config_json["name"], config_filename))
        return names

    @staticmethod
    def ls_files(config_dir) -> list[str]:
        files = []
        for file in os.listdir(config_dir):
            if file.endswith(".json"):
                files.append(os.path.join(config_dir, file))
        return files



import abc
import json
import os
from typing import Tuple

import click
from shell import shell

from .fs_config import FilesystemConfig


class FilesystemAbstract(abc.ABC):
    def __init__(self, config: FilesystemConfig):
        self.config = config
        self.hostname = config.hostname
        self.mount_point = config.mount_point
        self.user = config.user
        self.dir = config.dir
        self.options = config.options

        if config.remote:
            user, hostname, dir = self.parse_remote()

            if user:
                self.user = user
            if hostname:
                self.hostname = hostname
            if dir:
                self.dir = dir

    @abc.abstractmethod
    def parse_remote(self) -> Tuple[str, str, str]:
        raise NotImplementedError

    @abc.abstractmethod
    def format_remote(self) -> str:
        raise NotImplementedError

    @abc.abstractmethod
    def validate(self) -> bool:
        raise NotImplementedError

    @abc.abstractmethod
    def mount(self):
        raise NotImplementedError

    @staticmethod
    def unmount(mount_point):
        umount = shell(f"umount {mount_point}")
        os.rmdir(mount_point)



from typing import Tuple
import os

import click
from shell import Shell

from .fs_abc import FilesystemAbstract


class SMBFS(FilesystemAbstract):
    def parse_remote(self) -> Tuple[str, str, str]:
        remote_str = self.config.remote
        if not remote_str.startswith("//"):
            raise ValueError(f"Invalid samba path {remote_str}")

        remote_str = remote_str[2:]

        user, hostname, dir = "", "", ""

        if "@" in remote_str:
            user, hostname = remote_str.split("@")
        else:
            hostname = remote_str

        hostname, dir = hostname.split("/", 1)

        return user, hostname, dir

    def format_remote(self) -> str:
        remote_str = "//"
        if self.user:
            remote_str = f"{remote_str}{self.user}@"

        if not self.hostname:
            raise ValueError("host must not be None")
        remote_str = f"{remote_str}{self.hostname}/"

        if self.dir:
            remote_str = f"{remote_str}{self.dir}"
        return remote_str

    def format_cmd(self):
        remote_str = self.format_remote()

        if not self.mount_point:
            raise ValueError("mount_point must not be None")

        smbfs_cmd = f"mount_smbfs {remote_str} {self.mount_point}"
        return smbfs_cmd

    def validate(self):
        mount_cmd = self.format_cmd()
        if mount_cmd:
            return True
        else:
            return False

    def mount(self):
        if not os.path.exists(self.mount_point):
            os.makedirs(self.mount_point, exist_ok=True)
        mount_cmd = self.format_cmd()
        sh = Shell()
        sh.run(mount_cmd)



import click

# https://stackoverflow.com/questions/46641928/python-click-multiple-command-names
class Aliases(click.Group):
    def command(self, *args, **kwargs):
        """Behaves the same as `click.Group.command()` except if passed
        a list of names, all after the first will be aliases for the first.
        """

        def decorator(f):
            if isinstance(args[0], list):
                _args = [args[0][0]] + list(args[1:])
                for alias in args[0][1:]:
                    cmd = super(Aliases, self).command(alias, *args[1:], **kwargs)(f)
                    cmd.short_help = "Alias for '{}'".format(_args[0])
            else:
                _args = args
            cmd = super(Aliases, self).command(*_args, **kwargs)(f)
            return cmd

        return decorator




# -*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE!
# This file has been autogenerated by dephell <3
# https://github.com/dephell/dephell

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


import os.path

readme = ''
here = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(here, 'README.rst')
if os.path.exists(readme_path):
    with open(readme_path, 'rb') as stream:
        readme = stream.read().decode('utf8')


setup(
    long_description=readme,
    name='remote-fs',
    version='0.1.1',
    description='A tool to manage your remote filesystems (sshfs, mount_smbfs)',
    python_requires='==3.*,>=3.9.0',
    project_urls={"homepage": "https://github.com/kintsugi/remote-fs"},
    author='kintsu',
    author_email='k@kintsu.io',
    license='MIT',
    packages=['remote_fs', 'remote_fs.cli'],
    package_dir={"": "."},
    package_data={},
    install_requires=['click==7.*,>=7.1.2', 'pathvalidate==2.*,>=2.3.0', 'setuptools==50.*,>=50.3.0', 'shell==1.*,>=1.0.1'],
    extras_require={"dev": ["black==20.*,>=20.8.0.b1", "dephell==0.*,>=0.8.3", "neovim==0.*,>=0.3.1", "neovim-remote==2.*,>=2.4.0", "pylint==2.*,>=2.6.0", "pynvim==0.*,>=0.4.2", "rope==0.*,>=0.18.0", "twine==3.*,>=3.2.0"]},
)



import click

from .settings import Settings


class Context:
    def __init__(self, app_name, settings={"filesystem": ""}):
        self.app_name = app_name
        if not self.app_name:
            raise ValueError("app_name must not be empty")
        self.config_dir = click.get_app_dir(self.app_name)
        self.settings = Settings(**settings)



import sys

import click
from shell import CommandError

import remote_fs.fs as fs
from .ctx import Context
from remote_fs.fs_config import FilesystemConfig


@click.group()
@click.pass_context
@click.option(
    "--smbfs",
    "filesystem",
    flag_value="smbfs",
    default=False,
    help="mounts via mount_smbfs (darwin only)",
)
@click.option(
    "--sshfs", "filesystem", flag_value="sshfs", default=False, help="mounts via sshfs"
)
def cli(click_ctx, filesystem):
    click_ctx.obj = Context("remote-fs", settings={"filesystem": filesystem})


@cli.command()
@click.pass_context
@click.argument("mount_point", required=True)
@click.option("--load", is_flag=True, default=False)
@click.option("--name", required=False)
@click.option(
    "--remote", required=False, help="full hostname string, e.g. user@hostname:dir"
)
@click.option("--hostname", required=False)
@click.option("--user", required=False)
@click.option("--dir", required=False)
@click.option(
    "--option",
    "-o",
    multiple=True,
    default=["reconnect,ServerAliveInterval=15,ServerAliveCountMax=3"],
)
def mount(click_ctx, mount_point, load, name, **kwargs):
    ctx: Context = click_ctx.obj

    config = FilesystemConfig(
        filesystem=ctx.settings.filesystem, mount_point=mount_point, **kwargs
    )

    if name is not None:
        config.save(name, ctx.config_dir)
    elif load:
        config.load(mount_point, ctx.config_dir)
    else:
        raise ValueError("Name must be passed if not loading a config")

    filesystem = fs.create_fs(config)

    try:
        filesystem.mount()
        click.echo(f"mounted {config.mount_point}")
    except CommandError as e:
        click.echo(f"failed to mount {config.mount_point}: {e}", err=True)
        sys.exit(1)


@cli.command()
@click.pass_context
@click.argument("name")
def unmount(click_ctx, name):
    ctx: Context = click_ctx.obj
    configs = FilesystemConfig.ls_configs(ctx.config_dir)
    target_config: tuple[str, str] = None
    for config in configs:
        if config[0] == name:
            target_config = config

    if not target_config:
        raise ValueError(f"{name} is not a saved config")

    config = FilesystemConfig.from_file(target_config[1])
    try:
        fs.FilesystemAbstract.unmount(config.mount_point)
        click.echo(f"unmounted {config.mount_point}")
    except CommandError as e:
        click.echo(f"failed to unmount {config.mount_point}: {e}", err=True)
        sys.exit(1)


@cli.command()
@click.pass_context
@click.argument("resource")
def ls(click_ctx, resource):
    ctx: Context = click_ctx.obj
    if resource == "configs":
        for name, filename in FilesystemConfig.ls_configs(ctx.config_dir):
            click.echo(f'{name}\t"{filename}"')
    if resource == "mounts":
        fs.ls_mounts(ctx.config_dir)
